/*
/--------------------------------------------------------------------
|
|      $Id: FilterContrast.h,v 1.4 2001/01/13 14:44:52 uzadow Exp $
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#if !defined(INCL_FILTERCONTRAST)
#define INCL_FILTERCONTRAST

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Filter.h"

class CBmp;

//! Enhances or reduces the image contrast using a linear mapping
//! between input and output. The zero point (i. e., the intensity
//! that is neither enhanced nor reduced) has to be provided.
//! (A Contrast filter is defined in the following way: It lowers all
//! intensity values below a given threshold, and it raises them
//! beyond. Most applications position the threshold at 50 %. This
//! does not always yield the best results, especially if you have an
//! unbalanced dark/light ratio of pixels. The best you can do here
//! is play around with the threshold.)

class CFilterContrast : public CFilter
{
public:
  //! contrast is the slope of the function. offset is the intensity
  //! at which the color stays the same. Above this value, intensities
  //! are increased. Below it, they are reduced. With offset 128 and
  //! contrast 1, the image stays unchanged.
  CFilterContrast(double contrast,
                  BYTE offset);
  virtual ~CFilterContrast();
  virtual void Apply(CBmp *pBmpSource, CBmp* pBmpDest);

protected:
  double m_contrast;
  double m_offset;
};
#endif

/*
/--------------------------------------------------------------------
|
|      $Log: FilterContrast.h,v $
|      Revision 1.4  2001/01/13 14:44:52  uzadow
|      Doc update.
|
|      Revision 1.3  2001/01/12 23:30:01  uzadow
|      Doc update.
|
|      Revision 1.2  2001/01/12 23:04:37  uzadow
|      Comments.
|
|      Revision 1.1  2000/11/06 23:20:22  uzadow
|      Added Contrast, Intensity and Lightness filters by
|      Thomas Hirschmann
|
|
\--------------------------------------------------------------------
*/
