/*
/--------------------------------------------------------------------
|
|      $Id: FilterIntensity.cpp,v 1.2 2000/12/18 22:42:53 uzadow Exp $
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#include "stdpch.h"
#include "FilterIntensity.h"
#include "bitmap.h"
#include "anybmp.h"

CFilterIntensity::CFilterIntensity(double intensity, BYTE offset, double exponent)
{
  m_intensity = (double) intensity;
  m_offset    = (double) offset;
  m_exponent  = (double) exponent;
}

CFilterIntensity::~CFilterIntensity()
{

}

void CFilterIntensity::Apply(CBmp * pBmpSource, CBmp * pBmpDest) 
{
  double h, s, v;
  double intensityFactor;
  INT total = 0;
  register int inc;
  extern void fp_rgb_to_hsv(double* r, double* g, double *b);
  extern void fp_hsv_to_rgb(double* r, double* g, double *b);

  // double intensity = 4.0 * m_intensity - 200.0;
  double intensity = m_intensity-20;
  intensity /= 4.0;
  PLASSERT (pBmpSource->GetBitsPerPixel() >= 24);

  pBmpDest->Create (pBmpSource->GetWidth(), 
                    pBmpSource->GetHeight(),
                    pBmpSource->GetBitsPerPixel(),
                    pBmpSource->HasAlpha());

  BYTE ** pSrcLines = pBmpSource->GetLineArray();
  BYTE ** pDstLines = pBmpDest->GetLineArray();

  register int destWidth = pBmpDest->GetWidth();

  if(pBmpSource->GetBitsPerPixel() == 24)
   inc = 3;

  if(pBmpSource->GetBitsPerPixel() == 32)
   inc = sizeof(CPixel32);

  double csupp = intensity/pow(255.0, m_exponent);

  for (int y = 0; y < pBmpDest->GetHeight(); ++y)
  { // For each line
    BYTE * pSrcPixel = pSrcLines[y];
    BYTE * pDstPixel = pDstLines[y];

    for (register int x = 0; x < destWidth; ++x)
    {
       // Transform rgb->hsv
       h =(double) pSrcPixel[RGBA_RED];
       s =(double) pSrcPixel[RGBA_GREEN];
       v =(double) pSrcPixel[RGBA_BLUE];

       fp_rgb_to_hsv(&h, &s, &v);

       // Modify the intensity
       if (v >= m_offset)
        intensityFactor = 1.0 + csupp * pow((v-m_offset), m_exponent);
       else
        intensityFactor = 1.0;

       v *= intensityFactor;

       // Transform back to rgb.
       fp_hsv_to_rgb(&h, &s, &v);

       pDstPixel[RGBA_RED]   = (BYTE) h;
       pDstPixel[RGBA_GREEN] = (BYTE) s;
       pDstPixel[RGBA_BLUE]  = (BYTE) v;

       if(h >= 255.0)
         pDstPixel[RGBA_RED]   = (BYTE) 255;
       if(s >= 255.0)
         pDstPixel[RGBA_GREEN] = (BYTE) 255;
       if(v >= 255.0)
         pDstPixel[RGBA_BLUE]  = (BYTE) 255;
       if(h <= 0.0)
         pDstPixel[RGBA_RED]   = (BYTE) 0;
       if(s <= 0.0)
         pDstPixel[RGBA_GREEN] = (BYTE) 0;
       if(v <= 0.0)
         pDstPixel[RGBA_BLUE]  = (BYTE) 0;

       pSrcPixel += inc;
       pDstPixel += inc;
    }
  }
}

/*
/--------------------------------------------------------------------
|
|      $Log: FilterIntensity.cpp,v $
|      Revision 1.2  2000/12/18 22:42:53  uzadow
|      Replaced RGBAPIXEL with CPixel32.
|
|      Revision 1.1  2000/11/06 23:20:22  uzadow
|      Added Contrast, Intensity and Lightness filters by
|      Thomas Hirschmann
|
|
\--------------------------------------------------------------------
*/
