/*
/--------------------------------------------------------------------
|
|      $Id: FilterLightness.cpp,v 1.3 2001/01/12 23:29:38 uzadow Exp $
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#include "stdpch.h"
#include "FilterLightness.h"
#include "bitmap.h"
#include "anybmp.h"

CFilterLightness::CFilterLightness(int lightness)
{
  m_lightness = (double) lightness;
}

CFilterLightness::~CFilterLightness()
{

}

void CFilterLightness::Apply(CBmp * pBmpSource, CBmp * pBmpDest)
{
  int total = 0;

  double lightness = 2.55 * (double) (m_lightness);

  PLASSERT (pBmpSource->GetBitsPerPixel() == 32);

  pBmpDest->Create (pBmpSource->GetWidth(),
                    pBmpSource->GetHeight(),
                    pBmpSource->GetBitsPerPixel(),
                    pBmpSource->HasAlpha());

  BYTE ** pSrcLines = pBmpSource->GetLineArray();
  BYTE ** pDstLines = pBmpDest->GetLineArray();

  int destWidth = pBmpDest->GetWidth();

  double red, green, blue;

  for (int y = 0; y < pBmpDest->GetHeight(); ++y)
  { // For each line
    BYTE * pSrcPixel = pSrcLines[y];
    BYTE * pDstPixel = pDstLines[y];

    for (int x = 0; x < destWidth; ++x)
    {
       red   = (double) pSrcPixel[RGBA_RED] + lightness;
       green = (double) pSrcPixel[RGBA_GREEN] + lightness;
       blue  = (double) pSrcPixel[RGBA_BLUE] + lightness;

       if(red >= 255.0)
        pDstPixel[RGBA_RED] = (BYTE) 255;
       else if (red < 0.0)
        pDstPixel[RGBA_RED] = (BYTE) 0;
       else
        pDstPixel[RGBA_RED] = (BYTE) red;

       if(green >= 255.0)
        pDstPixel[RGBA_GREEN] = (BYTE) 255;
       else if (green < 0.0)
        pDstPixel[RGBA_GREEN] = (BYTE) 0;
       else
        pDstPixel[RGBA_GREEN] = (BYTE) green;

       if(blue >= 255.0)
        pDstPixel[RGBA_BLUE] = (BYTE) 255;
       else if (blue < 0.0)
        pDstPixel[RGBA_BLUE] = (BYTE) 0;
       else
        pDstPixel[RGBA_BLUE] = (BYTE) blue;

       pSrcPixel += 4;
       pDstPixel += 4;
    }
  }
}
