/*
/--------------------------------------------------------------------
|
|      $Id: FilterThreshold.cpp,v 1.2 2000/12/18 22:42:53 uzadow Exp $
|
|      Copyright (c) 2000-2001 Iyad Hatem
|
\--------------------------------------------------------------------
*/

#include "stdpch.h"
#include "FilterThreshold.h"
#include "bitmap.h"
#include "anybmp.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CFilterThreshold::CFilterThreshold(int threshold_min,int threshold_max, int channel)
: m_threshold_min(threshold_min),
  m_threshold_max(threshold_max),
  m_channel(channel)
{
}

CFilterThreshold::~CFilterThreshold()
{
	
}

void CFilterThreshold::Apply(CBmp * pBmpSource, CBmp * pBmpDest) 
{
	
	int threshold_min = m_threshold_min;
	int threshold_max = m_threshold_max;
	int channel = m_channel;
	
	// Only works for 32 bpp bitmaps at the moment.
	PLASSERT (pBmpSource->GetBitsPerPixel() == 32);
	
	pBmpDest->Create (pBmpSource->GetWidth(), pBmpSource->GetHeight(), 8, FALSE);
	BYTE ** pSrcLines = pBmpSource->GetLineArray();
	BYTE ** pDstLines = pBmpDest->GetLineArray();
	
	
	for (int y = 0; y<pBmpDest->GetHeight(); ++y)
	{ // For each line
		BYTE * pSrcPixel = pSrcLines[y];
		BYTE * pDstPixel = pDstLines[y];
		
		for (int x = 0; x < pBmpDest->GetWidth(); ++x)
		{ // For each pixel
			if ((BYTE (pSrcPixel[channel])<=BYTE (threshold_min)) || (BYTE (pSrcPixel[channel])>= BYTE(threshold_max)))
				*pDstPixel = BYTE (0);
			else 
				*pDstPixel = BYTE (pSrcPixel[channel]);
			pSrcPixel += sizeof(CPixel32);
			++pDstPixel;
		}
	}
}

/*
/--------------------------------------------------------------------
|
|      $Log: FilterThreshold.cpp,v $
|      Revision 1.2  2000/12/18 22:42:53  uzadow
|      Replaced RGBAPIXEL with CPixel32.
|
|      Revision 1.1  2000/09/26 14:28:47  Administrator
|      Added Threshold filter
|
|
|
\--------------------------------------------------------------------
*/
