/*
/--------------------------------------------------------------------
|
|      $Id: gifdec.cpp,v 1.3 2000/12/18 22:42:52 uzadow Exp $
|
|      TODO: Error handling.
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|      Initial version by Michael Salzlechner
|
\-------------------------------------------------------------------
*/

#include "stdpch.h"

#include "gifdec.h"
#include "anybmp.h"

extern "C" 
{
#define DOEXTERNGIF

// Workaround for namespace clash. This has been fixed in newer versions
// of libungif.
#ifdef _WINDOWS
#undef DrawText
#endif
#include "gif_lib.h"
}

static int
    InterlacedOffset[] = { 0, 4, 2, 1 }, /* The way Interlaced image should. */
    InterlacedJumps[] = { 8, 8, 4, 2 };    /* be read - offsets and jumps... */

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGIFDecoder::CGIFDecoder()
{
//	SetFileExtensions("*.gif");
//	SetFileDescription("GIF File");
}

CGIFDecoder::~CGIFDecoder()
{

}

int GIF_Read_Data(GifFileType* pGifFile,GifByteType* pByteType,int length)
{
  BYTE *ptr;
  CDataSource* pSourceInfo=(CDataSource*)pGifFile->UserData;

  ptr = pSourceInfo->ReadNBytes(length);
  memcpy(pByteType,ptr,length);

  return length;
}

void CGIFDecoder::DoDecode(CBmp *pBmp, CPixel32 **ppPal, int *pDestBPP, CDataSource *pDataSrc)
{
  GifFileType *GifFile;
  GifRecordType RecordType;
  GifByteType *Extension;
  ColorMapObject *ColorMap = NULL;

	int CurLine = 0;
	int i, Row, Col, Width, Height, Count, j, ExtCode;

	GifFile = DGifOpen( (void*)pDataSrc, GIF_Read_Data);

	pBmp->Create (GifFile->SWidth, GifFile->SHeight , 8, FALSE);

	BYTE ** pLineArray = pBmp->GetLineArray();

	CurLine = 0;
	while (CurLine < GifFile->SHeight)
	{
		pLineArray[0][CurLine] = GifFile->SBackGroundColor;
	    CurLine++;
	}

    /* Scan the content of the GIF file and load the image(s) in: */
  do
  {
    if (DGifGetRecordType(GifFile, &RecordType) == GIF_ERROR)
    {
	    PrintGifError();
      PLASSERT (FALSE);
	  }

    switch (RecordType)
    {
	    case IMAGE_DESC_RECORD_TYPE:
		    if (DGifGetImageDesc(GifFile) == GIF_ERROR)
        {
		      PrintGifError();
          PLASSERT (FALSE);
        }
        Row = GifFile->Image.Top; /* Image Position relative to Screen. */
        Col = GifFile->Image.Left;
        Width = GifFile->Image.Width;
        Height = GifFile->Image.Height;
        if (GifFile->Image.Left + GifFile->Image.Width > GifFile->SWidth ||
	          GifFile->Image.Top + GifFile->Image.Height > GifFile->SHeight)
        {
          fprintf(stderr, "Image %d is not confined to screen dimension, aborted.\n");
          PLASSERT (FALSE);
        }
        if (GifFile->Image.Interlace)
        {
		      /* Need to perform 4 passes on the images: */
		      for (Count = i = 0; i < 4; i++)
	          for (j = Row + InterlacedOffset[i]; j < Row + Height;
				         j += InterlacedJumps[i])
            {
			        GifQprintf("\b\b\b\b%-4d", Count++);
			        if (DGifGetLine(GifFile, &pLineArray[j][Col], Width) == GIF_ERROR)
              {
		            PrintGifError();
                PLASSERT (FALSE);
			        }
            }
        }
        else
        {
		      for (i = 0; i < Height; i++)
          {
			      GifQprintf("\b\b\b\b%-4d", i);
			      if (DGifGetLine(GifFile, &pLineArray[Row++][Col], Width) == GIF_ERROR)
            {
			        PrintGifError();
              PLASSERT (FALSE);
      			}
		      }
		    }
		    break;
	    case EXTENSION_RECORD_TYPE:
		    /* Skip any extension blocks in file: */
		    if (DGifGetExtension(GifFile, &ExtCode, &Extension) == GIF_ERROR)
        {
		      PrintGifError();
          PLASSERT (FALSE);
		    }
		    while (Extension != NULL)
        {
		      if (DGifGetExtensionNext(GifFile, &Extension) == GIF_ERROR)
          {
			      PrintGifError();
            PLASSERT (FALSE);
 		      }
		    }
		    break;
	    case TERMINATE_RECORD_TYPE:
		    break;
	    default:		    /* Should be trapped by DGifGetRecordType. */
		    break;
	  }
  } while (RecordType != TERMINATE_RECORD_TYPE);

  ColorMap = (GifFile->Image.ColorMap ? GifFile->Image.ColorMap :
	            GifFile->SColorMap);

	BYTE* pb = (BYTE*)ColorMap->Colors;
	for (i = 0; i < ColorMap->ColorCount; i++)
	{
		pBmp->SetPaletteEntry(i, pb[0], pb[1], pb[2], 255);
		pb += 3;
	}

	DGifCloseFile(GifFile);

  // create a 32 bpp bitmap if the caller requested one.
  if (*pDestBPP == 32)
  {
    CAnyBmp TempBmp;
    TempBmp.CreateCopy (*pBmp, 32);
    *pBmp = TempBmp;
  }
}

/*
/--------------------------------------------------------------------
|
|      $Log: gifdec.cpp,v $
|      Revision 1.3  2000/12/18 22:42:52  uzadow
|      Replaced RGBAPIXEL with CPixel32.
|
|      Revision 1.2  2000/12/17 20:45:57  uzadow
|      Fixed build problem with libungif.
|
|      Revision 1.1  2000/12/08 13:41:23  uzadow
|      Added gif support by Michael Salzlechner.
|
|
\--------------------------------------------------------------------
*/
