/*
/--------------------------------------------------------------------
|
|      $Id: pltester.cpp,v 1.10 2001/01/13 00:28:03 uzadow Exp $
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#include "stdpch.h"
#include "pltester.h"

#include <iostream>
#include <string>
#include <fstream>

#include "paintlibdefs.h"
#include "anybmp.h"
#include "anydec.h"
#include "bmpenc.h"
#include "bmpdec.h"
#include "jpegenc.h"
#include "jpegdec.h"
#include "tiffenc.h"
#include "tiffencex.h"
#include "tiffdec.h"
#include "pngenc.h"
#include "pngdec.h"
#include "filesink.h"

#include "Filter/Filter.h"
#include "Filter/FilterContrast.h"
#include "Filter/FilterCrop.h"
#include "Filter/FilterIntensity.h"
#include "Filter/FilterLightness.h"
#include "Filter/FilterQuantize.h"
#include "Filter/FilterResizeBilinear.h"
#include "Filter/FilterResizeGaussian.h"
#include "Filter/FilterRotate.h"


#ifdef _WINDOWS
#include "../../win/test/resource.h"
#endif

CPLTester::CPLTester ()
{
  
}

CPLTester::~CPLTester ()
{
}

void CPLTester::RunTests ()
{
  setUp();
  try
  {

    PLTRACE ("Testing Bmp classes...\n");
    PLTRACE ("  Testing 32 bpp CAnyBmp...\n");
    testCBmp (m_AnyBmp32);

    PLTRACE ("  Testing 8 bpp CAnyBmp...\n");
    testCBmp (m_AnyBmp8);

#ifdef _WINDOWS
    PLTRACE ("  Testing 32 bpp CDIBSection...\n");
    testCBmp (m_DIBSect32);
    testCWinBmp (m_DIBSect32);
    testDIBSect (m_DIBSect32);
    setUp();
    PLTRACE ("  Testing 8 bpp CDIBSection...\n");
    testCBmp (m_DIBSect8);
    testCWinBmp (m_DIBSect8);
    testDIBSect (m_DIBSect32);
    setUp();
    PLTRACE ("  Testing load from resource...\n");
    testLoadRes ();
#endif

#ifdef SUPPORT_BMP
    PLTRACE ("Testing Bmp Codec...\n");
    testCodec (CBmpEncoder(), CBmpDecoder());
#else
    PLTRACE ("Skipping Bmp Codec check - support not compiled into library.\n");
#endif
#ifdef SUPPORT_TIFF
    PLTRACE ("Testing TIFF Codec...\n");
    testCodec (CTIFFEncoder(), CTIFFDecoder());
    PLTRACE ("Testing TIFF Codec with compression...\n");
    PLTRACE ("  Testing 32 bpp CAnyBmp....\n");
    testCTIFFEx (m_AnyBmp32);
    PLTRACE ("  Testing 8 bpp CAnyBmp....\n");
    testCTIFFEx (m_AnyBmp8);
#else
    PLTRACE ("Skipping TIFF Codec check - support not compiled into library.\n");
#endif
#ifdef SUPPORT_PNG
    PLTRACE ("Testing PNG Codec...\n");
    testCodec (CPNGEncoder(), CPNGDecoder());
#else
    PLTRACE ("Skipping PNG Codec check - support not compiled into library.\n");
#endif   

    PLTRACE ("Testing Filters...\n");
    testFilters();

    // TODO:
    // - JPEG Codec test.
    // - Mode decoder tests from testdec.
    // - Codec tests using CMemSource and CMemSink.
    // - DirectDraw bmp tests.

  }
  catch (CTextException e)
  {
    PLTRACE ("\npaintlibtest: %s\n", (const char*)e);
    exit (1);
  }
  tearDown();
}

void CPLTester::testCBmp (CBmp& Bmp)
{
  {
    PLTRACE ("    Testing copy constructors...\n");
#ifdef _WINDOWS
    CWinBmp WinBmp (Bmp);
    test (Bmp == WinBmp);

    CDIBSection DIBSect (Bmp);
    test (Bmp == DIBSect);

#endif
    CAnyBmp AnyBmp (Bmp);
    test (Bmp == AnyBmp);
  }

  // Assignment operators
  {
    PLTRACE ("    Testing assignment operators...\n");
#ifdef _WINDOWS
    CWinBmp WinBmp;
    CDIBSection DIBSect;
    WinBmp = Bmp;
    test (Bmp == WinBmp);
    DIBSect = Bmp;
    test (Bmp == DIBSect);
#endif
    CAnyBmp AnyBmp;
    AnyBmp = Bmp;
    test (Bmp == AnyBmp);

    CAnyBmp * bmp2 = new CAnyBmp(Bmp);
  }

  // Misc. tests
  {
    if (Bmp.GetBitsPerPixel() == 8)
    {
      PLTRACE ("    Testing FindNearestColor()...\n");
      CPixel32 Pix = Bmp.GetPalette()[0];
      BYTE index = Bmp.FindNearestColor (Pix);
      test (index == 0);
    }
  }
}

#ifdef _WINDOWS
void CPLTester::testCWinBmp (CWinBmp& Bmp)
{
  // Test clipboard
  PLTRACE ("    Testing clipboard support....\n");
  Bmp.ToClipboard();

  CWinBmp DestBmp;
  DestBmp.FromClipboard();
// FromClipboard is broken at the moment.
//  PLASSERT (Bmp == DestBmp);
}

void CPLTester::testLoadRes ()
{
  // Load from resource
  CWinBmp ResBmp;
  ResBmp.CreateRes (NULL, IDB_TEST);
  // We can't make sure the data is loaded correctly because CreateRes
  // outputs a 24 bpp bitmap :-(.
}

void CPLTester::testDIBSect (CDIBSection& Bmp)
{
  // Get the handle and create a CWinBmp from it again.
  PLTRACE ("    Testing GetHandle()...\n");
  HBITMAP hBmp = Bmp.GetHandle ();
  CWinBmp TestBmp;
  TestBmp.CreateFromHBitmap (hBmp);
  test (TestBmp == Bmp);

  // Test if Detach() works.
  PLTRACE ("    Testing Detach()...\n");
  hBmp = Bmp.GetHandle();
  BITMAPINFOHEADER * pBMI = Bmp.GetBMI();
  Bmp.Detach();
  DeleteObject (hBmp);
  free (pBMI);
  PLASSERT_VALID (&Bmp);
}
#endif

void CPLTester::testCodec (const CPicEncoder& Encoder, const CPicDecoder& Decoder)
{
  PLTRACE ("  Testing 32 bpp CAnyBmp....\n");
  testCodecBmp (m_AnyBmp32, Encoder, Decoder, "test");
  PLTRACE ("  Testing 8 bpp CAnyBmp....\n");
  testCodecBmp (m_AnyBmp8, Encoder, Decoder, "test");
#ifdef _WINDOWS
  PLTRACE ("  Testing 32 bpp CDIBSection....\n");
  testCodecBmp (m_DIBSect32, Encoder, Decoder, "test");
  PLTRACE ("  Testing 8 bpp CDIBSection....\n");
  testCodecBmp (m_DIBSect8, Encoder, Decoder, "test");
#endif
}

void CPLTester::testCodecBmp (CBmp& Bmp, const CPicEncoder& Encoder, 
                              const CPicDecoder& Decoder, char * pszFName)
{
  CAnyBmp NewBmp;
  // paintlib isn't const-correct yet :-(.
  const_cast<CPicEncoder&>(Encoder).MakeFileFromBmp (pszFName, &Bmp);
  const_cast<CPicDecoder&>(Decoder).MakeBmpFromFile (pszFName, &NewBmp);
  ::remove (pszFName);  // Delete file again.
  test (Bmp == NewBmp);
}


void CPLTester::testCTIFFEx (CBmp& Bmp)
{
  const char szFName[] = "test";
  CAnyBmp NewBmp;
  CTIFFEncoderEx Encoder;
  CFileSink Sink;
  Sink.Open (szFName, Bmp.GetMemUsed() + 4096);
  Encoder.Associate (&Sink );
  // mandatory 'base_line' tags
  Encoder.SetBaseTags( &Bmp );
  // one strip = the whole image (no strip, no tile)
  Encoder.SetField( TIFFTAG_ROWSPERSTRIP, Bmp.GetHeight());
  Encoder.SetField( TIFFTAG_DOCUMENTNAME, szFName);
  Encoder.SetField (TIFFTAG_COMPRESSION, COMPRESSION_PACKBITS);
  Encoder.SaveBmp (&Bmp, &Sink);
  // only really needed if you encode several bitmaps
  Encoder.Dissociate ();
  Sink.Close();

  CTIFFDecoder Decoder;
  Decoder.MakeBmpFromFile (szFName, &NewBmp);
  ::remove (szFName);  // Delete file again.
  test (Bmp == NewBmp);
}


void CPLTester::testFilters ()
{
  PLTRACE ("  Testing contrast...\n");
  testFilter (CFilterContrast (1, 128));

  PLTRACE ("  Testing crop...\n");
  testFilter (CFilterCrop (0,5,0,5));

  PLTRACE ("  Testing intensity...\n");
  testFilter (CFilterIntensity (20, 128, 1));

  PLTRACE ("  Testing lightness...\n");
  testFilter (CFilterLightness (0));

  PLTRACE ("  Testing quantization...\n");
  PLTRACE ("    Testing median cut...\n");
  testQuant (DTHPAL_MEDIAN, DTH_NONE);
  PLTRACE ("    Testing popularity sort...\n");
  testQuant (DTHPAL_POPULARITY, DTH_NONE);
  PLTRACE ("    Testing default palette...\n");
  testQuant (DTHPAL_DEFAULT, DTH_NONE);
  PLTRACE ("    Testing ordered dithering...\n");
  testQuant (DTHPAL_MEDIAN, DTH_ORDERED);
  PLTRACE ("    Testing floyd-steinberg dithering...\n");
  testQuant (DTHPAL_MEDIAN, DTH_FS);

  PLTRACE ("  Testing bilinear resize...\n");
  testFilter (CFilterResizeBilinear (5,5));

  PLTRACE ("  Testing gauss filter...\n");
  testFilter (CFilterResizeGaussian (5,5,0.2));

  PLTRACE ("  Testing rotation...\n");
  testFilter (CFilterRotate (0, CPixel32 (0,0,0,0)));  
  testFilter (CFilterRotate (2*3.141592, CPixel32 (0,0,0,0)));  
}

void CPLTester::testFilter (const CFilter& Filter)
{
  CAnyBmp DestBmp;
  const_cast<CFilter&>(Filter).Apply (&m_AnyBmp32, &DestBmp);
  test (almostEqual (m_AnyBmp32, DestBmp, 4));
}  

void CPLTester::testQuant (int DthPal, int Dth)
{
  CAnyBmp DestBmp8;
  CAnyBmp DestBmp32;
  CFilterQuantize Q (DthPal, Dth);
  Q.Apply(&m_AnyBmp32, &DestBmp8);
  DestBmp32.CreateCopy (DestBmp8, 32);
  test (almostEqual (m_AnyBmp32, DestBmp32, 4));
}

void CPLTester::setUpBmp (CBmp& Bmp, int BPP)
{
  Bmp.Create(5, 5, BPP, false);
  BYTE ** ppLines = Bmp.GetLineArray();
  int y,x;
  for (y=0; y<5; y++)
    for (x=0; x<5; x++)
    {
      CPixel32 * pPix;
      switch (BPP)
      {
        case 8:
          if (y<2)
            ppLines[y][x] = 255;
          else if (y<3)
            ppLines[y][x] = 128;
          else
            ppLines[y][x] = 0;
          break;
        case 32:
          pPix = (CPixel32*)&(ppLines[y][x*4]);
          if (y == 0)
            pPix->Set(255,255,255,0);
          else if (y == 1)
            pPix->Set(255,255,0,0);
          else if (y == 2 || y == 3)
            pPix->Set(255,0,0,0);
          else if (y == 4)
            pPix->Set(0,0,0,0);
          break;
        default:
          PLASSERT (false);
      }
    }
}

void CPLTester::setUp ()
{
  setUpBmp (m_AnyBmp32, 32);
  setUpBmp (m_AnyBmp8, 8);
#ifdef _WINDOWS
  setUpBmp (m_DIBSect32, 32);
  setUpBmp (m_DIBSect8, 8);
//  setUpBmp (m_DDBmp32, 32);
//  setUpBmp (m_DDBmp8, 8);
#endif
}

void CPLTester::tearDown ()
{
}

void CPLTester::dumpBmp (CBmp& bmp)
{
  PLTRACE ("  Size: %ix%i\n", bmp.GetWidth(), bmp.GetHeight());
  PLTRACE ("  Pixel data: \n");

  BYTE ** ppLines = bmp.GetLineArray();
  int y,x;
  for (y=0; y<bmp.GetHeight(); y++)
  {
    PLTRACE ("    ");
    for (x=0; x<bmp.GetWidth(); x++)
    {
      CPixel32 pix;
      switch (bmp.GetBitsPerPixel())
      {
        case 8:
          PLTRACE ("%i, ", ppLines[y][x]);
          break;
        case 32:
          pix = *((CPixel32*)&(ppLines [y][x*4]));
          PLTRACE ("(%i,%i,%i,%i) ", pix.GetR(), pix.GetG(), pix.GetB (), pix.GetA ());
          break;
        default:
          PLASSERT (false);
      }
    }
    PLTRACE ("\n");
  }
  PLTRACE ("\n");
}

bool CPLTester::almostEqual (CBmp& bmp1, CBmp& bmp2, int epsilon)
{
  if (bmp1.GetWidth() != bmp2.GetWidth() ||
      bmp1.GetHeight() != bmp2.GetHeight() ||
      bmp1.HasAlpha() != bmp2.HasAlpha() ||
      bmp1.GetBitsPerPixel() != bmp2.GetBitsPerPixel())
    return false;

  BYTE ** ppLines1 = bmp1.GetLineArray();
  BYTE ** ppLines2 = bmp2.GetLineArray();
  int y,x;
  for (y=0; y<bmp1.GetHeight(); y++)
    for (x=0; x<bmp1.GetWidth(); x++)
      switch (bmp1.GetBitsPerPixel())
      {
        case 8:
          if (abs (ppLines1[y][x] - ppLines2[y][x]) > epsilon)
            return false;
          break;
        case 32:
          if (abs (ppLines1[y][x*4+RGBA_RED] - ppLines2[y][x*4+RGBA_RED]) > epsilon ||
              abs (ppLines1[y][x*4+RGBA_GREEN] - ppLines2[y][x*4+RGBA_GREEN]) > epsilon ||
              abs (ppLines1[y][x*4+RGBA_BLUE] - ppLines2[y][x*4+RGBA_BLUE]) > epsilon)
            return false;
          if (bmp1.HasAlpha() &&
              abs (ppLines1[y][x*4+3] - ppLines2[y][x*4]+3) > epsilon)
            return false;
          break;
        default:
          // Bitmap must be either 8 or 32 bpp for this routine to work.
          PLASSERT (false);
      }
  return true;
}

void CPLTester::test (bool b)
{
  if (!b)
    PLTRACE ("           ---->> failed\n");
// Comment in to stop when an assertion fails
// PLASSERT (b);  
}

/*
/--------------------------------------------------------------------
|
|      $Log: pltester.cpp,v $
|      Revision 1.10  2001/01/13 00:28:03  uzadow
|      no message
|
|      Revision 1.9  2001/01/12 23:06:11  uzadow
|      Additional bmp assignment test.
|
|      Revision 1.8  2000/12/18 22:42:53  uzadow
|      Replaced RGBAPIXEL with CPixel32.
|
|      Revision 1.7  2000/12/09 12:16:26  uzadow
|      Fixed several memory leaks.
|
|      Revision 1.6  2000/12/05 00:12:18  uzadow
|      *** empty log message ***
|
|      Revision 1.5  2000/12/04 23:59:02  uzadow
|      Added filter tests.
|
|      Revision 1.4  2000/12/03 21:15:43  uzadow
|      Fixed png handling in configure; test/ adapted to linux
|
|      Revision 1.3  2000/12/02 19:42:29  uzadow
|      Added CTIFFEncoderEx test.
|
|      Revision 1.2  2000/11/21 23:28:53  uzadow
|      Moved CBmp tests from testdec to test.
|
|      Revision 1.1  2000/11/21 20:29:39  uzadow
|      Added test project.
|
|
\--------------------------------------------------------------------
*/
